function [heave_aft_port heave_aft_starboard heave_front_starboard heave_front_port] = compute_cargo_deck_corners(roll_pitch_yaw, position, time, pos, ang)

% Preallocate storage
real_time = position(:, 1);
measure_time = time - 30;

real_delta_time = (real_time(length(real_time)) - real_time(1)) / length(real_time);
len = floor(measure_time(length(measure_time)) / real_delta_time);
real_time = real_time(1:len);

heave_aft_port = zeros(len, 2);
heave_aft_starboard = zeros(len, 2);
heave_front_starboard = zeros(len, 2);
heave_front_port = zeros(len, 2);

deck_width = 0.1801;
deck_length = 0.3849;

aft_port = [-deck_length / 2 -deck_width / 2 0]';
aft_starboard = [-deck_length / 2 deck_width / 2 0]';
front_starboard = [deck_length / 2 deck_width / 2 0]';
front_port = [deck_length / 2 -deck_width / 2 0]';

real_pos = zeros(len, 3);
measure_pos = zeros(len, 3);

for i=1:len
    % Real
    roll = roll_pitch_yaw(i, 2) * pi / 180;
    pitch = roll_pitch_yaw(i, 3) * pi / 180;
    yaw = roll_pitch_yaw(i, 4) * pi / 180;
    R_b_n = Rzyx(roll, pitch, yaw);
    p = position(i, 2:4)';
    real_pos(i, :) = p;
    
    v = p + R_b_n * aft_port;
    heave_aft_port(i, 1) = v(3);
     
    v = p + R_b_n * aft_starboard;
    heave_aft_starboard(i, 1) = v(3);
     
    v = p + R_b_n * front_starboard;
    heave_front_starboard(i, 1) = v(3);
    
    v = p + R_b_n * front_port;
    heave_front_port(i, 1) = v(3);
    
    % Measured (interpolated to match real_time(i))   
    roll = interp1(measure_time, ang(:, 1), real_time(i));
    pitch = interp1(measure_time, ang(:, 2), real_time(i));
    yaw = interp1(measure_time, ang(:, 3), real_time(i));
    R_b_n = Rzyx(roll, pitch, yaw);
    
    p = [   interp1(measure_time, pos(:, 1), real_time(i));
            interp1(measure_time, pos(:, 2), real_time(i));
            interp1(measure_time, pos(:, 3), real_time(i))];
   
    measure_pos(i, :) = p;
    
    v = p + R_b_n * aft_port;
    heave_aft_port(i, 2) = v(3);
     
    v = p + R_b_n * aft_starboard;
    heave_aft_starboard(i, 2) = v(3);
     
    v = p + R_b_n * front_starboard;
    heave_front_starboard(i, 2) = v(3);
     
    v = p + R_b_n * front_port;
    heave_front_port(i, 2) = v(3);
end



