function [R, t] = compute_attitude(actuator)

global a_1 a_2 a_3 a_min r d

% Length is actuator minimum plus the measured value
a_1 = a_min + actuator(1);
a_2 = a_min + actuator(2);
a_3 = a_min + actuator(3);

% Actuators will always be close to vertical so this is a nice start point
x0 = [pi/2; pi/2; pi/2];
[x, fval] = fsolve(@geofun, x0);    
%x = [pi/2 pi/2 pi/2];

alpha_1 = x(1);
alpha_2 = x(2);
alpha_3 = x(3);

% Optimize
c_120 = cos(120 * pi / 180);
c_240 = cos(240 * pi / 180);
s_120 = sin(120 * pi / 180);
s_240 = sin(240 * pi / 180);

% Compute the position of the points
p_1 = [ 0;
        r + a_1 * cos(alpha_1);
        a_1 * sin(alpha_1) ];

p_2 = [ s_120 * (r + a_2 * cos(alpha_2));
        c_120 * (r + a_2 * cos(alpha_2));
        a_2 * sin(alpha_2) ];

p_3 = [ s_240 * (r + a_3 * cos(alpha_3));
        c_240 * (r + a_3 * cos(alpha_3));
        a_3 * sin(alpha_3) ];

% Compute the points in the platform frame which defined relative to the 
% fixed platform frame
%R_r_n = [0  -1   0;
%         -1  0   0;
%         0  0   -1];

% Compute difference vectors for convenience
p_3_1 = p_1 - p_3;
p_2_3 = p_3 - p_2;

y_b = p_2_3 ./ norm(p_2_3);
z_b = cross(y_b, p_3_1) / norm(cross(y_b, p_3_1));
x_b = cross(y_b, z_b);

R_p_f = [x_b y_b z_b];
R = R_p_f;

t = (p_1 + p_2 + p_3) ./ 3;
    
end




