% Compute position and attitude of motion platform based on actuator
% positions

clear

global a_min d r

load('platform_log.mat')

% Motion platform parameters
a_min = 0.363;
d = 0.432;
a_middle = 0.10;

% Ensures that actuators are vertical when platform is level
r = d / (sin(60 / 180 * pi) * 2)

bottom_to_deck = 0.096;

% Center of cargo deck relative platform center given in the platform frame
actuator_top_hinge_height = 0.06;
cargo_deck_center = [-0.182; 0; -bottom_to_deck - actuator_top_hinge_height];

% Center of calibration origin given in the actuator frame
calibration_center = [0.053; 0.364; a_min + a_middle + bottom_to_deck + actuator_top_hinge_height]


% Converting indexes
start = 1;
%len = 2;
len = length(platform_log);
roll_pitch_yaw = zeros(len - start, 4);
position = zeros(len - start, 4);
t_all = zeros(len - start, 3);

for i = start:len
    actuator = platform_log(i, 8:10) ./ 1000;
    [R_p_f, t] = compute_attitude(actuator);
    t_all(i, :) = t';
    R_f_n = [ 0  -1   0;
              -1  0   0;
              0   0   -1];
    
    % Deck center given in the actuator frame
    deck_center_a = t + R_p_f * cargo_deck_center;
    
    % Deck center given in the camera world
    deck_center_w = R_f_n * (deck_center_a - calibration_center);
    
    position(i, :) = [platform_log(i, 1) deck_center_w'];
        
    R_p_n = R_f_n * R_p_f;
    yaw = atan2(R_p_n (1,2) , R_p_n (1,1)) * 180 / pi;
    pitch = -asin(R_p_n (3,1)) * 180 / pi;
    roll = atan2(R_p_n (3,2), R_p_n (3,3)) * 180 / pi;
    roll_pitch_yaw(i, :) = [platform_log(i, 1) roll pitch yaw];
end

clear R_f_n R_p_f R_p_n a_middle a_min actuator actuator_top_hinge_height ...
    ans bottom_to_deck cargo_deck_center d deck_center_a deck_center_w ...
    i len pitch r roll start t t_all yaw calibration_center

save('platform_pos_attitude.mat', 'position', 'roll_pitch_yaw');
disp('Output written to: platform_pos_attitude.mat');



