% Plot figure 9.2


clear
load('video_measurements/regular_tracking.mat')
load('motion_platform_measurements/platform_pos_attitude.mat')

time_min = -5;
time_max = 250;

video_start = 30;
real = roll_pitch_yaw;
measured = [(time - video_start) ang * 180 / pi];

first_time = measured(1,1);
last_time = measured(length(measured),1);
delta_time = (last_time - first_time) / length(measured);
start_idx = (time_min - first_time) / delta_time;
end_idx = (time_max - first_time) / delta_time;

if start_idx < 1
    start_idx = 1
end
if end_idx > length(measured)
    end_idx = length(measured);
end

measured_clip = measured(700:end_idx, :);
real_clip = real(start_idx:end_idx, :);
scale1 = max([measured_clip(:,2); real_clip(:,2)]) - min([measured_clip(:,2); real_clip(:,2)]);
scale2 = max([measured_clip(:,3); real_clip(:,3)]) - min([measured_clip(:,3); real_clip(:,3)]);
scale3 = max([measured_clip(:,4); real_clip(:,4)]) - min([measured_clip(:,4); real_clip(:,4)]);
scale = max([scale1 scale2 scale3]) * 1.05;


first_plots_stretch = 0.75;
plot_no = 0;
num_plots = 3;
plot_height_step = (first_plots_stretch / 3);
plot_height = plot_height_step - 0.08;

figure('Position', [855 400 550 830]);

subp = subplot(4, 1, 1);
hold on
idx = 2;
plot(measured(:,1), measured(:,idx), 'k')
plot(real(:,1), real(:,idx), 'r');
xlabel('time [s]');
ylabel('angle [deg]');
mean_val = mean([measured_clip(:, idx); real_clip(:, idx)]);
axis([time_min time_max mean_val - scale / 2 mean_val + scale / 2]);
set(subp,'Position', [0.1, 1 - (plot_no + 1) * plot_height_step + 0.025, .8, plot_height ]);
leg = legend('Measured', 'Real');
set(leg,'FontSize',8);
title('Roll');

subp = subplot(4, 1, 2);
hold on
plot_no = plot_no + 1;
idx = 3;
plot(measured(:,1), measured(:,idx) , 'k');
plot(real(:,1), real(:,idx), 'r');
set(subp,'Position', [0.1, 1 - (plot_no + 1) * plot_height_step + 0.025, .8, plot_height ]);
xlabel('time [s]');
ylabel('angle [deg]');
title('Pitch');
mean_val = mean([measured_clip(:, idx); real_clip(:, idx)]);
axis([time_min time_max mean_val - scale / 2 mean_val + scale / 2]);

subp = subplot(4, 1, 3);
hold on
plot_no = plot_no + 1;
idx = 4;
plot(measured(:,1), measured(:,idx), 'k');
set(subp,'Position', [0.1, 1 - (plot_no + 1) * plot_height_step + 0.025, .8, plot_height ]);
plot(real(:,1), real(:,idx), 'r');
xlabel('time [s]');
ylabel('angle [deg]');
title('Yaw');
mean_val = mean([measured_clip(:, idx); real_clip(:, idx)]);
axis([time_min time_max mean_val - scale / 2 mean_val + scale / 2]);

last_plot_stretch = 1 - first_plots_stretch;
subp = subplot(4, 1, 4);
plot(time - video_start, num_points, 'b');
set(subp,'Position', [0.1, 0.1, 0.8, last_plot_stretch - 0.16]);
set(subp, 'TickDir', 'out', 'YTick', 1:2:10);
xlabel('time [s]');
ylabel('points [1]');
title('Tracked 3D points');
axis([time_min time_max 0 10]);

%set(gcf, 'PaperPositionMode', 'Auto');
%print -painters -depsc -r300 ../../thesis/figures/6_meter_roll_pitch_yaw.eps
%system('epstopdf ../../thesis/figures/6_meter_roll_pitch_yaw.eps');