% Plot figure 9.5 a, b, c, d

clear
load('cargo_deck_corners_comparison/platform_time.mat')
load('cargo_deck_corners_comparison/corners_regular.mat')
load('cargo_deck_corners_comparison/corners_subpixel.mat')

% High-pass filter
f_s = 25;
f_cut = 1; % Cut off frequency
f_norm = f_cut / (f_s / 2);
[b, a] = butter(5, f_norm, 'high');

% Histogram parameters
sidelobe = 0.003;
bins = 50;
hist_bins = [0:2 * sidelobe / bins: 2 * sidelobe] - sidelobe;

accuracy_idx_start = floor(bins / 2 - 0.001 / (2 * sidelobe / bins));
accuracy_idx_stop = ceil(bins / 2 + 0.001 / (2 * sidelobe / bins) + 1);

normal = [heave_aft_port_normal(:,2) heave_aft_starboard_normal(:,2) heave_front_starboard_normal(:,2) heave_front_port_normal(:,2)];
subpixel = [heave_aft_port_subpixel(:,2) heave_aft_starboard_subpixel(:,2) heave_front_starboard_subpixel(:,2) heave_front_port_subpixel(:,2)];
normal_len = length(normal);
subpixel_len = length(subpixel);

names = ['aft_port', 'aft_stbd', 'fnt_stbd', 'fnt_port'];

for corner=1:4
   
    figure('Position', [30 + corner * 300 300 260 380])
    subp = subplot(3, 1, 1);
    set(subp,'Position', [0.18, 0.68, 0.77, 0.26]);
    hold on
    plot(real_time(1:normal_len), normal(:,corner), 'k');
    plot(real_time(1:subpixel_len), subpixel(:,corner), 'b');
    title('heave','FontSize',8);
    xlabel('time [t]','FontSize',8);
    ylabel('position [m]','FontSize',8);
    axis([100 150 -0.03 0.06]);
    set(gca,'FontSize',8');
    leg = legend('regular', 'subpixel');
    set(leg,'FontSize',8', 'Orientation', 'horizontal');
    
    name = sprintf('%s',names((corner - 1) * 8 + 1:corner * 8));

    subp = subplot(3, 1, 2);
    set(subp,'Position', [0.18, 0.40, 0.77, 0.15]);
    noise = filter(b, a, normal(:,corner));
    y = hist(noise, hist_bins);
    bar(hist_bins, y, 'FaceColor', [0 0 0], 'EdgeColor', 'none');
    xlim([-sidelobe * ( 1 - 1/bins) sidelobe * ( 1 - 1/bins)]);
    title('error distribution regular','FontSize',8)
    xlabel('error [m]','FontSize',8);
    ylabel('samples [1]','FontSize',8);
    set(gca,'FontSize',8');

    disp(['Probability regular ', name, ':'])
    sum(y(accuracy_idx_start:accuracy_idx_stop)) / sum(y)
    
    subp = subplot(3, 1, 3);
    set(subp,'Position', [0.18, 0.12, 0.77, 0.15]);
    noise = filter(b, a, subpixel(:,corner));
    y = hist(noise, hist_bins);
    bar(hist_bins, y, 'FaceColor', [0 0 0], 'EdgeColor', 'none');
    title('error distribution 1/3 subpixel','FontSize',8);
    xlim([-sidelobe * ( 1 - 1/bins) sidelobe * ( 1 - 1/bins)]);
    xlabel('error [m]','FontSize',8);
    ylabel('samples [1]','FontSize',8);
    set(gca,'FontSize',8');

    disp(['Probability subpixel ', name, ':'])
    sum(y(accuracy_idx_start:accuracy_idx_stop)) / sum(y)

    %set(gcf, 'PaperPositionMode', 'Auto') ;
    %filename =  sprintf('../../thesis/figures/heave_%s.eps', name);
    %print('-painters','-depsc','-r300', filename);
    %system(['epstopdf ../../thesis/figures/heave_', name, '.eps']);
    %delete(['../../thesis/figures/heave_', name, '.eps']);
    
end



