#include "calibrationdialog.h"
#include "ui_calibrationdialog.h"
#include <QDebug>
#include <QStringList>



CalibrationDialog::CalibrationDialog(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::CalibrationDialog)
{
    m_ui->setupUi(this);

    int rows = 8;
    m_ui->tableWidget->setRowCount(rows);
    QStringList list;

    CalibrationPoint defaultPoint;
    defaultPoint.x = 0;
    defaultPoint.y = 0;
    defaultPoint.z = 0;
    defaultPoint.u = 0;
    defaultPoint.v = 0;

    for(int i = 0; i < rows; i++) {
        points.append(defaultPoint);
        QString str;
        str.setNum(i);
        list << str;
        QTableWidgetItem * item;
        item = new QTableWidgetItem(str.setNum(defaultPoint.x));
        item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        m_ui->tableWidget->setItem(i, 0, item);
        m_ui->tableWidget->setItem(i, 1, new QTableWidgetItem(*item));
        m_ui->tableWidget->setItem(i, 2, new QTableWidgetItem(*item));
        m_ui->tableWidget->setItem(i, 3, new QTableWidgetItem(*item));
        m_ui->tableWidget->setItem(i, 4, new QTableWidgetItem(*item));
    }

    CalibrationPoint point0 = {0, 0, 0, 0, 0};
    setCalibrationPoint(0, point0);

    CalibrationPoint point1 = {0, 0.11, 0, 0, 0};
    setCalibrationPoint(1, point1);

    CalibrationPoint point2 = {0.11, 0.11, 0, 0, 0};
    setCalibrationPoint(2, point2);

    CalibrationPoint point3 = {0.11, 0, 0, 0, 0};
    setCalibrationPoint(3, point3);

    CalibrationPoint point4 = {0, 0, -0.11, 0, 0};
    setCalibrationPoint(4, point4);

    CalibrationPoint point5 = {0, 0.11, -0.11, 0, 0};
    setCalibrationPoint(5, point5);

    CalibrationPoint point6 = {0.11, 0.11, -0.11, 0, 0};
    setCalibrationPoint(6, point6);

    CalibrationPoint point7 = {0.11, 0, -0.11, 0, 0};
    setCalibrationPoint(7, point7);

    m_ui->tableWidget->setVerticalHeaderLabels(list);
    m_ui->tableWidget->setCurrentCell(0, 0);
}

CalibrationDialog::~CalibrationDialog()
{
    delete m_ui;
}

void CalibrationDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void CalibrationDialog::on_buttonBox_accepted()
{

}

void CalibrationDialog::setCalibrationPoint(int row, CalibrationPoint point)
{
    QString str;
    m_ui->tableWidget->item(row, 0)->setText(str.setNum(point.x));
    m_ui->tableWidget->item(row, 1)->setText(str.setNum(point.y));
    m_ui->tableWidget->item(row, 2)->setText(str.setNum(point.z));
    m_ui->tableWidget->item(row, 3)->setText(str.setNum(point.u));
    m_ui->tableWidget->item(row, 4)->setText(str.setNum(point.v));
    if (row < points.size() - 1) {
        m_ui->tableWidget->setCurrentCell(row + 1, 0);
    }

    points.replace(row, point);
}

void CalibrationDialog::pointSelected(QPoint point)
{
    int currentRow = m_ui->tableWidget->currentRow();
    CalibrationPoint newPoint = points.at(currentRow);
    newPoint.u = point.x();
    newPoint.v = point.y();
    this->setCalibrationPoint(currentRow, newPoint);
}

QList<CalibrationPoint> * CalibrationDialog::getCalibrationPoints()
{
    return &points;
}


