#include "cameracontrols.h"
#include "ui_cameracontrols.h"
#include "camera.h"
#include "vesselsearch.h"
#include "calibrationdialog.h"
#include "imageprocessing.h"
#include "houghsvg.h"

#include <QFileDialog>
#include <QDebug>
CameraControls::CameraControls(Camera * camera) : ui(new Ui::CameraControls)
{
    ui->setupUi(this);

    this->camera = camera;
    calibrationDialog = NULL;
    connect(ui->clearAllTrackers, SIGNAL(clicked()), camera->getImageProcessing(), SLOT(removeAllTrackers()));
    connect(ui->vesselSearchButton, SIGNAL(clicked()), camera->getImageProcessing(), SLOT(setVesselSearch()));
    connect(ui->calibrateButton, SIGNAL(clicked()), camera, SLOT(showCalibrationDialog()));
    houghSVG = new HoughSVG();

}

Camera * CameraControls::getParent()
{
    return camera;
}

CameraControls::~CameraControls()
{
    delete houghSVG;
    delete ui;
}

void CameraControls::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void CameraControls::updateTrackerList(void)
{
    QVector <int> ids = camera->getImageProcessing()->getTrackerIds();
    ui->listWidget->clear();

    for(int i = 0; i < ids.size(); i++) {
        QString num;
        num.append("Tracker ");
        num.setNum(ids.at(i));
        ui->listWidget->addItem(num);
    }
}


void CameraControls::updateVideoInfo()
{
    QString src = QString("Video source: ") + camera->getVideoSourceDescription();
    ui->videoSourceLabel->setText(tr("...") + src.right(32));

    ui->cameraName->setText(QString("Camera \"") + camera->getName() + tr("\""));
}

void CameraControls::on_listWidget_clicked(QModelIndex index)
{
    camera->getImageProcessing()->removeTracker(ui->listWidget->item(index.row())->text().toInt());
}

void CameraControls::on_setCheckImage_toggled(bool checked)
{
    camera->setCheckImageMode(checked);
}


void CameraControls::on_houghSvgButton_clicked()
{
    houghSVG->setLines(camera->getImageProcessing()->getVesselSearch()->getLines(), camera->getImageProcessing()->getVesselSearch()->getWorldLines(), 100);

    if (ui->houghSvgPreview->checkState() == Qt::Checked) {
        houghSVG->showOriginal();
    } else {
        QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "hough_lines.svg", tr("Scalable Vector Graphics (*.svg)"));
        if (!fileName.isNull()) {
            houghSVG->saveSvg(fileName, QSize(camera->getFrameWindow().width(), camera->getFrameWindow().height()));
        }
    }
}

void CameraControls::on_houghWorldSvgButton_clicked()
{
    houghSVG->setLines(camera->getImageProcessing()->getVesselSearch()->getLines(), camera->getImageProcessing()->getVesselSearch()->getWorldLines(), 100);

    if (ui->houghWorldSvgPreview->checkState() == Qt::Checked) {
        houghSVG->showWorld();
    } else {
        QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "hough_world_lines.svg", tr("Scalable Vector Graphics (*.svg)"));
        if (!fileName.isNull()) {
            houghSVG->saveWorldSvg(fileName, QSize(camera->getFrameWindow().width(), camera->getFrameWindow().height()));
        }
    }
}

void CameraControls::on_saveFrameButton_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "hough_world_lines.svg", tr("Portable Network Graphics (*.png)"));
    if (!fileName.isNull()) {
        cvSaveImage(fileName.toAscii().data(), camera->getFrame());
    }
}


void CameraControls::on_saveFilteredFrameButton_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "hough_world_lines.svg", tr("Portable Network Graphics (*.png)"));
    if (!fileName.isNull()) {
        cvSaveImage(fileName.toAscii().data(), camera->getImageProcessing()->getFilteredFrameCV());
    }
}

void CameraControls::on_showLines_toggled(bool checked)
{
    camera->getImageProcessing()->getVesselSearch()->setLinesVisible(checked);
}

void CameraControls::on_removeButton_clicked()
{
    emit removeCamera();
}

void CameraControls::on_loadCamMatrix_clicked()
{
    QString filename = QFileDialog::getOpenFileName(this, tr("Open File"), ".", tr("OpenCV data storage (*.xml *.yml *.yaml)"));
    if (!filename.isNull()) {
        camera->loadCameraMatrix(filename);
    }
}
