#ifndef CAMERAMANAGER_H
#define CAMERAMANAGER_H

#include <QObject>
#include <QThread>
#include <QSemaphore>
#include <QTimer>
#include <QTime>
#include <QMutex>
#include "camera.h"
#include "export.h"

class TrackingManager;
class PositionEstimator;
class WaveEstimator;


class CameraManager : public QThread
{
Q_OBJECT
public:
    explicit CameraManager(QObject *parent = 0);
    ~CameraManager();
    void addCamera(Camera *);
    void removeCamera(Camera *);
    void setTrackingManager(TrackingManager *);
    void setPositionEstimator(PositionEstimator *);
    void setWaveEstimator(WaveEstimator *);
    float getSampleRate();
    bool getUnderRate();
    bool finishThread();
    void frameProcessed(QString);
    void run();

private:
    QMutex mutex;
    QVector <Camera *> cameras;
    QSemaphore * busyCameras;
    QSemaphore threadFinished;
    QTime time;
    QTimer * timer;
    float sampleRate;
    int counter;
    int framesInProcessing;
    int numAvgSampleRate;
    float maxSampleRate;
    int sumProcessingTime;
    bool underRate;
    bool endThread;
    TrackingManager * trackingManager;
    PositionEstimator * positionEstimator;
    WaveEstimator * waveEstimator;
    bool deleteCameras();
    int frameNo;
    Export exportObj;
    bool doExport;

signals:
    void allFramesProcessed();
    void exportInfo(int points, int size);

public slots:
    void startExport();
    void stopExport();
    void saveLog(QString fileName);

};

#endif // CAMERAMANAGER_H
