#include "export.h"
#include "mat.h"

#include <QDebug>

Export::Export()
{
    num = 0;
}

void Export::clear()
{
    time.clear();
    position.clear();
    angle.clear();
    state.clear();
    numTrackingPoints.clear();
    num++;
}

void Export::addData(float newTime, QVector3D pos, QVector3D ang, int stateId, int newNumTrackingPoints)
{
    time.append(newTime);
    position.append(pos);
    angle.append(ang);
    state.append(stateId);
    numTrackingPoints.append(newNumTrackingPoints);
}

void Export::writeToMat(QString fileName )
{
    MATFile * outMat = matOpen(fileName.toAscii().data(), "w");

    if (outMat == NULL) {
        qDebug() << "Write MAT-file failed";
        return;
    }

    int length = this->time.size();

    int status;
    mxArray * time = mxCreateDoubleMatrix(length, 1, mxREAL);
    mxArray * pos = mxCreateDoubleMatrix(length, 3, mxREAL);
    mxArray * ang = mxCreateDoubleMatrix(length, 3, mxREAL);
    mxArray * state = mxCreateNumericMatrix(length, 1, mxINT8_CLASS, mxREAL);
    mxArray * numPoints = mxCreateNumericMatrix(length, 1, mxINT8_CLASS, mxREAL);

    double * timePtr = (double *) mxGetPr(time);
    double * posPtr = (double *) mxGetPr(pos);
    double * angPtr = (double *) mxGetPr(ang);
    unsigned char * statePtr = (unsigned char  *) mxGetPr(state);
    unsigned char * numPointsPtr = (unsigned char *) mxGetPr(numPoints);

    

    // MATLAB uses column-major order
    for(int i = 0; i < length; i++) {

        timePtr[i] = this->time.at(i);

        posPtr[i] = position.at(i).x();
        posPtr[i + length] = position.at(i).y();
        posPtr[i + 2 * length] = position.at(i).z();

        angPtr[i] = angle.at(i).x();
        angPtr[i + length] = angle.at(i).y();
        angPtr[i + 2 * length] = angle.at(i).z();

        statePtr[i] = this->state.at(i);
        numPointsPtr[i] = numTrackingPoints.at(i);
    }

    if (matPutVariable(outMat, "time", time) != 0) {
        qDebug() << "Error using matPutVariable";
    }
    if (matPutVariable(outMat, "pos", pos) != 0) {
        qDebug() << "Error using matPutVariable";
    }
    if (matPutVariable(outMat, "ang", ang) != 0) {
        qDebug() << "Error using matPutVariable";
    }
    if (matPutVariable(outMat, "state", state) != 0) {
        qDebug() << "Error using matPutVariable";
    }
    if (matPutVariable(outMat, "num_points", numPoints) != 0) {
        qDebug() << "Error using matPutVariable";
    }

    mxDestroyArray(time);
    mxDestroyArray(pos);
    mxDestroyArray(ang);
    mxDestroyArray(state);
    mxDestroyArray(numPoints);


    matClose(outMat);
}

int Export::getNumDataPoints()
{
    return time.size();
}

int Export::getDataSize()
{
    return time.size() * sizeof(double) * 5 + 2 * sizeof(int);
}
