#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>

#include <QTimer>
#include <QTime>
#include <QFile>
#include <QErrorMessage>
#include <QVector>
#include <time.h>

#include "camera.h"
#include "videoscene.h"
#include "positionestimator.h"
#include "plot.h"
#include "cameramanager.h"
#include "trackingmanager.h"
#include "parameters.h"
#include "export.h"

#ifdef Q_OS_LINUX
class Sys1394;
#endif
class Presentation;
class WaveEstimator;
class SourceSelector;


namespace Ui
{
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    QGraphicsScene *graphicsScene;
    Camera * addCamera(QString);

private:
    Ui::MainWindow *ui;

    clock_t start;
    bool trackingViewInitied;
    IplImage * frame;
    int play;
    QTimer * timer;
    QTime time;
    int simult;
    QVector <Camera *> cameras;

    PositionEstimator * positionEstimator;
    void printMatrix(CvMat * L);
    void storeMatrix(CvMat * L, QString filename);
    CvMat * retrieveMatrix(QString filename);

    QFile * file;
    void closeEvent ( QCloseEvent * event );
    bool captureToFile;
    Plot * plot;
    char counter;
    int framesInProcessing;
    bool processVideo;
    QMutex displayMutex;
    VideoScene * videoScene;
    CameraManager * cameraManager;
#ifdef Q_OS_LINUX
    Sys1394 * sys1394;
#endif
    bool showVideo;
    int trackerId;
    QVector3D angles;
    TrackingManager * trackingManager;
    IplImage * patch;
    Presentation * presentation;
    Parameters * parameters;
    void updatePresentation();
    WaveEstimator * waveestimator;
    bool doUpdateUi;
    void initPresentation();
    SourceSelector * sourceSelector;
    bool addCameraFromSource(VideoSourceType type, QString id, QString name);
    void removeCamera(Camera * camera);
    void updateMenuBar();
    QMenu * sourcesMenu;

public slots:
    void cameraRemoveRequest();

private slots:
    void on_showPresentation_clicked();
    void on_showVideoButton_toggled(bool checked);
    void on_horizontalSlider_sliderMoved(int position);
    void on_actionNew_triggered();
    void on_stopCaptureButton_clicked();
    void on_startCaptureButton_clicked();
    void writeSettings();
    void readSettings();
    void updateUi();
    void scheduleUpdateUi();
    void rememberTrackerId(int);
    void addTrackerGroup(int);
    void pointSelected(QPoint);
    void vesselPolygonsFound(QVector <QPolygonF>);
    void sourceSelectorClosed(int result);

};

#endif // MAINWINDOW_H
