#include "mapselector.h"
#include <QPainter>
#include <QDebug>
#include <QResizeEvent>

Mapselector::Mapselector(QWidget *parent)
{
    gridSize = QSize(5, 7);
    minSquareSize = 20;
    selectedSquare = QPoint(0, 0);
    setMinimumSize(gridSize * minSquareSize);
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
}

void Mapselector::setSize(QSize size)
{
    gridSize = size;
    repaint();
}

void Mapselector::mousePressEvent(QMouseEvent * event)
{
    QRect paintArea = gridRect();
    if (paintArea.contains(event->pos())) {
        QPoint gridPos = event->pos() - paintArea.topLeft();
        int x = gridPos.x() * gridSize.width() / paintArea.size().width();
        int y = gridPos.y() * gridSize.height() / paintArea.size().height();
        selectedSquare = QPoint(x, y);
        squareSelected(selectedSquare);
        repaint();
    }

}

void Mapselector::mouseMoveEvent(QMouseEvent * event)
{


}

void Mapselector::resizeEvent(QResizeEvent * event)
{

}

QRect Mapselector::gridRect()
{
    int squareSizeX = size().width() / gridSize.width();
    int squareSizeY = size().height() / gridSize.height();

    int squareSize = minSquareSize;

    if (squareSizeX > squareSizeY) {
        squareSize = squareSizeY;
    } else {
        squareSize = squareSizeX;
    }

    int left = size().width() / 2.0f - squareSize * gridSize.width() / 2.0f;
    return QRect(left, 0, squareSize * gridSize.width(), squareSize * gridSize.height());
}

void Mapselector::paintEvent(QPaintEvent * event)
{
    QPainter painter(this);
    painter.setBrush(QBrush(QColor(255, 100, 100)));
    painter.setPen(QPen(QColor(0, 0, 0)));

    QRect paintArea = gridRect();
    int squareSize = (paintArea.width() - 1) / gridSize.width();

    for(int i = 0; i < gridSize.width(); i++) {
        for(int j = 0; j < gridSize.height(); j++) {
            painter.drawRect(QRect(paintArea.left() + i * squareSize, paintArea.top() + j * squareSize, squareSize, squareSize));
        }
    }

    painter.setPen(QPen(QColor(255, 220, 220)));

    painter.setBrush(QBrush(QColor(255, 200, 200)));
    painter.drawRect(QRect(paintArea.left() + selectedSquare.x() * squareSize, paintArea.top() + selectedSquare.y() * squareSize, squareSize, squareSize));
}

QSize Mapselector::sizeHint() const
{
    return (gridSize * minSquareSize) + QSize(1, 1);
}
