#include "plot.h"
#include "ui_plot.h"
#include <QDebug>
#include <QFileDialog>

Plot::Plot(QWidget *parent) :
    QWidget(parent),
    m_ui(new Ui::Plot)
{
    m_ui->setupUi(this);
    plotX = new QwtPlot();
    plotX->setTitle(QString("X"));
    //plotX->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);


    plotY = new QwtPlot();
    plotY->setTitle(QString("Y"));
    //plotY->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    plotZ = new QwtPlot();
    plotZ->setTitle(QString("Z"));
    //plotZ->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);


    xCurve = new QwtPlotCurve("Position");
    xCurve->setStyle(QwtPlotCurve::Steps);
    xCurve->setPen(QPen(QColor(200, 0, 0)));

    yCurve = new QwtPlotCurve("Position");
    yCurve->setStyle(QwtPlotCurve::Steps);
    yCurve->setPen(QPen(QColor(200, 0, 0)));

    zCurve = new QwtPlotCurve("Position");
    zCurve->setStyle(QwtPlotCurve::Steps);
    zCurve->setPen(QPen(QColor(200, 0, 0)));


    //plotX->setAxisScale(0, -10, 10, 0);
    //plotY->setAxisScale(0, -10, 10, 0);
    //plotZ->setAxisScale(0, -10, 10, 0);

    plotX->setAxisScale(2, 0, 0 + interval, 0);
    plotY->setAxisScale(2, 0, 0 + interval, 0);
    plotZ->setAxisScale(2, 0, 0 + interval, 0);

    interval = 20;
    leftTime = 0;

    m_ui->horizontalLayout->addWidget(plotX);
    m_ui->horizontalLayout->addWidget(plotY);
    m_ui->horizontalLayout->addWidget(plotZ);


    xCurve->attach(plotX);
    yCurve->attach(plotY);
    zCurve->attach(plotZ);

    replot = 0;

    exportNumDataPoints = 0;
    exportDataSize = 0;
}

Plot::~Plot()
{
    delete m_ui;
}

void Plot::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void Plot::clear()
{
    time.clear();
    x.clear();
    y.clear();
    z.clear();

    plotX->setAxisScale(2, 0, 0 + interval, 0);
    plotY->setAxisScale(2, 0, 0 + interval, 0);
    plotZ->setAxisScale(2, 0, 0 + interval, 0);

    leftTime = 0;
    xMax = xMin = yMax = yMin = zMax = zMin = 0;

}

void Plot::setPositions(QVector <QVector3D> positions)
{
    
    m_ui->positions->clear();

    for(int i = 0; i < positions.size(); i++) {
        QString val;
        QString num;
        val += "[";
        num.setNum(positions.at(i).x());
        val.append(num);
        val += ",";
        num.setNum(positions.at(i).y());
        val.append(num);
        val += ",";
        num.setNum(positions.at(i).z());
        val.append(num);
        val += "]";
        m_ui->positions->addItem(val);
    }

}

void Plot::setExportInfo(int numDataPoints, int dataSize)
{
    this->exportDataSize = dataSize;
    this->exportNumDataPoints = numDataPoints;
}

void Plot::showExportInfo()
{
    m_ui->numSaved->setText(QString::number(exportNumDataPoints));
    m_ui->dataSize->setText(QString::number(exportDataSize) + QString(" bytes"));
}

void Plot::setPos(double timeVal, QVector3D pos, QVector3D attitude)
{
    time.append(timeVal);

    float xVal = pos.x();
    float yVal = pos.y();
    float zVal = pos.z();

    QString num;
    num.setNum(xVal, 'f', 4);
    num.append(" m");
    m_ui->xPos->setText(num);
    num.setNum(yVal, 'f', 4);
    num.append(" m");
    m_ui->yPos->setText(num);
    num.setNum(zVal, 'f', 4);
    num.append(" m");
    m_ui->zPos->setText(num);

    num.setNum(attitude.z() * 180 / M_PI, 'f', 1);
    num.append(" deg");
    m_ui->yaw->setText(num);
    num.setNum(attitude.y() * 180 / M_PI, 'f', 1);
    num.append(" deg");
    m_ui->pitch->setText(num);
    num.setNum(attitude.x() * 180 / M_PI, 'f', 1);
    num.append(" deg");
    m_ui->roll->setText(num);

    x.append(xVal);
    y.append(yVal);
    z.append(zVal);

    if (xVal > xMax) {
        xMax = xVal;
        plotX->setAxisScale(0, xMin, xMax, 0);
    } else if (xVal < xMin) {
        xMin = xVal;
        plotX->setAxisScale(0, xMin, xMax, 0);
    }


    if (yVal > yMax) {
        yMax = yVal;
        plotY->setAxisScale(0, yMin, yMax, 0);
    } else if (yVal < yMin) {
        yMin = yVal;
        plotY->setAxisScale(0, yMin, yMax, 0);
    }

    if (yVal > yMax) {
        yMax = yVal;
        plotY->setAxisScale(0, yMin, yMax, 0);
    } else if (yVal < yMin) {
        yMin = yVal;
        plotY->setAxisScale(0, yMin, yMax, 0);
    }

    if (zVal > zMax) {
        zMax = zVal;
        plotZ->setAxisScale(0, zMin, zMax, 0);
    } else if (zVal < zMin) {
        zMin = zVal;
        plotZ->setAxisScale(0, zMin, zMax, 0);
    }

    xCurve->setData(time, x);
    yCurve->setData(time, y);
    zCurve->setData(time, z);

    if (timeVal > leftTime + interval) {

        plotX->setAxisScale(2, timeVal, timeVal + interval, 0);
        plotY->setAxisScale(2, timeVal, timeVal + interval, 0);
        plotZ->setAxisScale(2, timeVal, timeVal + interval, 0);

        time.clear();
        x.clear();
        y.clear();
        z.clear();

        leftTime = timeVal;
    }

    replot++;

    if((replot % 1) == 0) {
        this->plotX->replot();
        this->plotY->replot();
        this->plotZ->replot();
    }
}

void Plot::on_startLogging_clicked()
{
    emit startLogging();
    m_ui->startLogging->setDisabled(true);
    m_ui->stopLogging->setEnabled(true);
}

void Plot::on_stopLogging_clicked()
{
    emit stopLogging();
    m_ui->startLogging->setEnabled(true);
    m_ui->stopLogging->setDisabled(true);
}

void Plot::on_saveToMat_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "export.mat", tr("MAT-files (*.mat)"));
    if (!fileName.isNull()) {
        emit saveToMatFile(fileName);
    }

}
