#ifndef PLOT_H
#define PLOT_H

#include <QtGui/QWidget>
#include <QVector>
#include <QVector3D>

#include <qwt_painter.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_curve.h>
#include <qwt_scale_widget.h>
#include <qwt_legend.h>
#include <qwt_scale_draw.h>
#include <qwt_math.h>
#include "qwt_plot.h"


namespace Ui {
    class Plot;
}

class Plot : public QWidget {
    Q_OBJECT
public:
    Plot(QWidget *parent = 0);
    void setPos(double time, QVector3D position, QVector3D attitude);
    void setPositions(QVector <QVector3D> positions);
    void clear();
    void showExportInfo();
    ~Plot();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::Plot *m_ui;
    QwtPlot * plotX;
    QwtPlot * plotY;
    QwtPlot * plotZ;

    QwtPlotCurve * xCurve;
    QwtPlotCurve * yCurve;
    QwtPlotCurve * zCurve;

    QVector <double> time;
    QVector <double> x;
    QVector <double> y;
    QVector <double> z;

    double leftTime;
    double interval;

    double xMin, xMax, yMin, yMax, zMin, zMax;

    int replot;

    /* Display variables for export */
    int exportNumDataPoints;
    int exportDataSize;


public slots:
    void setExportInfo(int numDataPoints, int dataSize);

signals:
    void startLogging();
    void stopLogging();
    void saveToMatFile(QString fileName);

private slots:


private slots:
    void on_saveToMat_clicked();
    void on_stopLogging_clicked();
    void on_startLogging_clicked();
};

#endif // PLOT_H
