#ifndef PRESENTATION_H
#define PRESENTATION_H

#include <QWidget>
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QVector3D>

class VideoFrame;
class Wavemap;
class QwtPlot;
class QwtPlotCurve;
class Mapselector;

namespace Ui {
    class Presentation;
}

class Presentation : public QWidget {
    Q_OBJECT
public:
    Presentation(QWidget *parent = 0);
    ~Presentation();
    VideoFrame * getVideoFrame(int num);
    void resizeEvent(QResizeEvent * even);
    Wavemap * getWavemap();
    void setDeckPosition(QVector3D pos, QVector3D ang);

    VideoFrame * getFrameLeft();
    VideoFrame * getFrameRight();
    VideoFrame * getFrameTop();
    void setWavemap(float time, float * z, float * amplitude, QSize squares, float squareSize);
    void setRollPitch(float roll, float pitch, float z);
    void initStorage(QSize size);
    void setDeckVisible(bool visible);
    void setTrackingInfo(QString text);
    void setScale(QString unit, float scale, float plotDeltaTick);
    void clear();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::Presentation *ui;
    QGraphicsScene * topScene;
    QGraphicsScene * leftScene;
    QGraphicsScene * rightScene;
    QGraphicsScene * vesselScene;
    QGraphicsScene * logoScene;

    VideoFrame * topFrame;
    VideoFrame * leftFrame;
    VideoFrame * rightFrame;
    Wavemap * wavemap;
    QwtPlot * plot;
    QwtPlotCurve * plotCurve;
    void resizeItem(QGraphicsItem * item, QSize viewSize);
    void scaleView(QRect itemSize, QGraphicsView * view);
    Mapselector * mapselector;
    QPoint selectedSquare;
    QwtPlotCurve * heaveCurve;
    QVector <double> timeData;   
    QVector <QVector <double> > heaveData;
    QSize mapSize;
    float timeInterval;
    float scaleFactor;
    QString unit;
    float plotDeltaTick;
    bool directionUp;

    QGraphicsPixmapItem aftView;
    QGraphicsPixmapItem sideView;
    QGraphicsPixmapItem logoItem;
    float prevSquareHeight;
    float derivative;
    void lowpass(float input, float &output, float timeconstant);
    bool unitsInFeet;

public slots:
    void changeSelectedSquare(QPoint square);


private slots:
    void on_toggleUnit_clicked();
};

#endif // PRESENTATION_H
