#include "sys1394.h"

#include <QDebug>
#include <highgui.h>
#include "video1394.h"



Sys1394::Sys1394()
{
    inited = false;
    init();
    firstCamera = true;
    // cvNamedWindow("video1");
    // cvNamedWindow("video2");

}

Sys1394::~Sys1394()
{
    dc1394_camera_free_list(list);
    dc1394_free(d);
}

bool Sys1394::init()
{
    video_mode = (dc1394video_mode_t) 0;
    d = dc1394_new ();

    if (!d) {
        qDebug() << "Unable to start IIDC 1394 subsystem.";
        return false;
    }

    err = dc1394_camera_enumerate (d, &list);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    if (list->num == 0) {
        dc1394_log_error("No cameras found");
        return false;
    } else {
        qDebug() << "GUID list is: ";
        for(int i = 0; i < list->num; i++) {
            qDebug() << list->ids[i].guid;
        }
    }

    return true;
}

/**
  * Opens camera with specified GUID
  */
Video1394 * Sys1394::openCamera(uint64_t guid)
{
    bool found = false;
    int camIndex = 0;
    for(int i = 0; i < list->num; i++) {
        if (list->ids[i].guid == guid) {
            found = true;
            camIndex = i;
        }
    }

    if (found == false) {
        qDebug() << "IEEE 1394 device with GUID " << guid << " was not found";
        qDebug() << "Available devices are: ";
        for(int i = 0; i < list->num; i++) {
            qDebug() << i << " GUID: " << list->ids[i].guid;
        }
        return NULL;
    }

    dc1394camera_t * camera = dc1394_camera_new(d, list->ids[camIndex].guid);

    if (!camera) {
        qDebug() << "Could not open camera";
        return false;
    }

    if (firstCamera == true) {
        //dc1394_reset_bus(camera);
        firstCamera = false;
    }

    err = dc1394_video_set_operation_mode(camera, DC1394_OPERATION_MODE_1394B);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    qDebug() << "Using camera with GUID " << camera->guid;

    dc1394framerates_t framerates;
    dc1394video_modes_t video_modes;

    /*-----------------------------------------------------------------------
     *  get the best video mode and highest framerate. This can be skipped
     *  if you already know which mode/framerate you want...
     *-----------------------------------------------------------------------*/
    // get video modes:
    err=dc1394_video_get_supported_modes(camera,&video_modes);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    // select highest res mode:
    int i = 0;
    for (i = video_modes.num - 1; i >= 0; i--) {
        if (!dc1394_is_video_mode_scalable(video_modes.modes[i])) {
            dc1394_get_color_coding_from_video_mode(camera, video_modes.modes[i], &coding);
            if (coding == DC1394_COLOR_CODING_MONO8) {
                video_mode = video_modes.modes[i];
                break;
            }
        }
    }

    if (i < 0) {
        dc1394_log_error("Could not get a valid MONO8 mode");
        return false;
    }

    err = dc1394_get_color_coding_from_video_mode(camera, video_mode,&coding);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    // get highest framerate
    err = dc1394_video_get_supported_framerates(camera,video_mode,&framerates);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }
    framerate = framerates.framerates[framerates.num-1];

    /*-----------------------------------------------------------------------
     *  setup capture
     *-----------------------------------------------------------------------*/
    err = dc1394_video_set_iso_speed(camera, DC1394_ISO_SPEED_800);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    err = dc1394_video_set_mode(camera, video_mode);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    err = dc1394_video_set_framerate(camera, framerate);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    err = dc1394_capture_setup(camera, 4, DC1394_CAPTURE_FLAGS_DEFAULT);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    qDebug() << "continue";

    QString num;
    num.setNum(id);

    Video1394 * videoCamera = new Video1394(num);
    videoCamera->set1394Camera(camera);
    cameras.append(videoCamera);
    return videoCamera;
}



