#ifndef TRACKINGMANAGER_H
#define TRACKINGMANAGER_H

#include <QObject>
#include <QVector>
#include <QLine>
#include <QPolygon>
#include <QTime>
#include <QVector3D>

class Camera;
class PositionEstimator;
class VesselSearch;

enum TrackingState {SEARCHING, FINDING_AFT, ISOLATING_AFT, ADDING_TRACKERS, IN_TRACKING};

class TrackingManager : public QObject
{
    Q_OBJECT;
public:
    TrackingManager();
    void setPositionEstimator(PositionEstimator *);
    void addCamera(Camera *);
    void removeCamera(Camera *);
    void checkTracking();
    TrackingState getTrackingState();

private:
    QVector <Camera *> cameras;
    PositionEstimator * positionEstimator;
    QLineF computeEpipolarLine(Camera * camera1, QPointF point, Camera * camera2);
    QVector <int> findUntrackedPolygons(Camera * camera, QVector <QPolygon> polygonsImage);
    QVector <int> findTrackedPolygons(Camera * camera, QVector <QPolygon> polygonsImage);
    QTime timeSinceTrackingChange;
    int addCorrespondingTracker(Camera * firstCam, int trackerId, Camera * secondCam, QPolygonF worldPolygon);
    QVector <int> findTrackersInPositionGroup(Camera * camera);
    QVector <int> findStableTrackers(Camera * camera, int secondsOld, QVector <int> except);
    int findPolygonFromPosition(QVector <QPolygon> polygons, QPointF position);
    int addTrackersToUntrackedAreas(Camera * camera, QVector <QPolygon> polygons, int aftestIdx);
    void addTrackers(Camera * camera, QVector <QPolygon> polygons);
    void addTrackersToAft(Camera * camera, QVector <QPolygon> polygons);
    QVector <QPoint> findGoodFeaturesInPolygons(Camera * camera, QVector <QPolygon> polygons, int maxPoints);
    int findTrackerGroupByTracker(Camera * camera, int trackerId);
    void doFindCorrespondence(Camera * camera, QVector <QPolygon> polygonsImage, QVector <QPolygonF> polygonsWorld);
    void setInitialPos(QVector2D forwardVec, QPointF centerLine);
    void runPendingVesselFound();


    QVector <int> stableTrackers;
    int distanceToClosestTracker(Camera * camera, QPoint point);
    bool adjustWithCheckImage();

    bool addTrackerToPrimary;
    bool findCorrespondence;
    bool findInitialPos;

    float aftToBodyOrigin;
    int primaryCamera;
    int secondaryCamera;
    bool camerasOk;
    bool inTracking;
    float minMatch;
    TrackingState trackingState;
    int aftestTrackerId;
    int successiveAftNotInPolygon;
    QVector3D aftestPointBodyOffset;
    bool aftestPointBodyOffsetSet;
    int adjustWithCheckImageTimer;
    float maxLockOffset;
    bool startVesselSearch;

    bool doAdjustWithCheckImage;
    bool vesselHasBeenFound;
    VesselSearch * vesselSearch;

public slots:
    void vesselFound(QLine, QLine);

signals:
    void newVesselPolygons(QVector <QPolygonF> polygons);

};

#endif // TRACKINGMANAGER_H

