#include "video1394.h"

#include <QDebug>
Video1394::Video1394(QString id)
{
    frameRotated = NULL;
    rotateFrame = false;
}

void Video1394::setRotated()
{
    rotateFrame = true;
}

Video1394::~Video1394()
{
    err = dc1394_video_set_transmission(camera, DC1394_OFF);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
    }
    cvReleaseImageHeader(&cvFrame);
}

bool Video1394::set1394Camera(dc1394camera_t * camera)
{
    this->camera = camera;
}

bool Video1394::init() {

    err = dc1394_video_set_transmission(camera, DC1394_ON);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    err = dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_WAIT, &frame);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }
    cvFrame = cvCreateImageHeader(cvSize(frame->size[0], frame->size[1]), IPL_DEPTH_8U, 1);
    return true;
}

bool Video1394::captureFrame()
{
    err = dc1394_capture_enqueue(camera, frame);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    err = dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_WAIT, &frame);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
        return false;
    }

    cvFrame->imageData = (char *) frame->image;
    return true;
}


QSize Video1394::getVideoSize()
{
    if (rotateFrame) {
        return QSize(cvFrame->height, cvFrame->width);
    }
    return QSize(cvFrame->width, cvFrame->height);
}

IplImage * Video1394::getFrame()
{
    if (rotateFrame) {
        if (frameRotated == NULL) {
            frameRotated = cvCreateImage(cvSize(cvFrame->height, cvFrame->width), IPL_DEPTH_8U, 1);
        }
        cvTranspose(cvFrame, frameRotated);
        cvFlip(frameRotated, frameRotated);
        return frameRotated;
    }

    return cvFrame;
}

void Video1394::close()
{
    err = dc1394_video_set_transmission(camera, DC1394_OFF);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
    }
    err = dc1394_capture_stop(camera);
    if (err) {
        qDebug() << QString(dc1394_error_get_string(err));
    }
    dc1394_camera_free(camera);
}
