#include <QDebug>
#include <QPen>
#include <QRectF>
#include <QGraphicsSceneMouseEvent>

#include "videoframe.h"
#include "videographicsitem.h"

VideoFrame::VideoFrame() : QGraphicsItemGroup()
{

    rectangle = new QGraphicsRectItem(this);
    rectangle->setPen(QPen(Qt::black));
    rectangle->setOpacity(0.2);
    rectangle->setBrush(QBrush (Qt::red));
    rectangle->setVisible(false);
    rectangle->setRect(0, 0, 100, 100);

    osdText.setDefaultTextColor(QColor(255, 255, 255));

    QFont font(QString("verdana"), 42);
    font.setStyleStrategy(QFont::PreferAntialias);
    osdText.setFont(font);
    osdText.setZValue(5);
    osdText.setOpacity(1.0);
    addToGroup(&osdText);

    osdTextShadow.setDefaultTextColor(QColor(0, 0, 0));
    osdTextShadow.setFont(QFont(QString("verdana"), 42));
    osdTextShadow.setZValue(4);
    osdTextShadow.setPos(osdText.pos() + QPointF(2, 2));
    osdTextShadow.setOpacity(0.7);
    addToGroup(&osdTextShadow);

    polygon = new QGraphicsPolygonItem();
    polygon->setOpacity(0.2);
    polygon->setBrush(QBrush(QColor(0, 0, 255)));
    polygon->setZValue(2);
    polygon->setVisible(false);
    QVector <QPointF> vector;
    vector.append(QPointF(0, 0));
    vector.append(QPointF(100, 0));
    vector.append(QPointF(100, 100));
    vector.append(QPointF(0, 100));
    polygon->setPolygon(QPolygonF(vector));
    crosshairH = new QGraphicsLineItem(this);
    QPen pen = QPen(QColor(255, 255, 0));
    pen.setWidth(5);
    crosshairH->setPen(pen);
    crosshairH->setZValue(2);

    for(int i = 0; i < 20; i++) {
        QGraphicsRectItem * crosshair = new QGraphicsRectItem(this);
        crosshair->setPen(pen);
        crosshair->setZValue(2);
        this->crosshairs.push_back(crosshair);
        addToGroup(crosshair);
    }

    addToGroup(rectangle);
    addToGroup(crosshairH);
    frameDecoration.setZValue(1);
    frameDecoration.setPen(QPen(Qt::gray));
    addToGroup(&frameDecoration);
    addToGroup(polygon);
    videoItem = new VideoGraphicsItem(this);

    videoItem->setZValue(0);

    addToGroup(videoItem);
    rectangle->setZValue(1);

    data = NULL;
}

void VideoFrame::setOSD(QString text)
{
    osdText.setPlainText(text);
    osdTextShadow.setPlainText(text);
}

void VideoFrame::setCameraMatrix(CvMat * m)
{
    videoItem->setCameraMatrix(m);
}

void VideoFrame::setPolygon(QPolygon poly)
{
    //polygon->setPolygon(poly);
    //polygon->setVisible(true);
}

void VideoFrame::setLine(QLine line)
{
    this->crosshairH->setLine(line);
    crosshairH->setVisible(true);

}

void VideoFrame::setFrameCV(IplImage * frame)
{
    videoItem->setFrame(frame);
    videoSize.setWidth(frame->width);
    videoSize.setHeight(frame->height);
    frameDecoration.setRect(QRect(0, 0, videoSize.width(), videoSize.height()));
}

void VideoFrame::pressEvent(QGraphicsSceneMouseEvent * event)
{
    QPointF pos = mapFromParent(event->pos());

    if (event->button() == Qt::LeftButton) {
        topLeft = bottomRight = pos.toPoint();
        emit pointSelected(pos.toPoint());
    }
}


void VideoFrame::moveEvent ( QGraphicsSceneMouseEvent * event )
{
    QPointF pos = mapFromParent(event->pos());

    bottomRight = pos.toPoint();
    rectangle->setVisible(true);

    QRect bounding = videoItem->boundingRect().toRect();

    if (bottomRight.x() < 0) {
        bottomRight.setX(0);
    }
    if (bottomRight.y() < 0) {
        bottomRight.setY(0);
    }

    if (bottomRight.x() >= bounding.width() - 2) {
        bottomRight.setX(bounding.width() - 2);
    }
    if (bottomRight.y() >= bounding.height() - 2) {
        bottomRight.setY(bounding.height() - 2);
    }

    rectangle->setRect(QRect(topLeft, bottomRight));
}

bool VideoFrame::contains ( const QPointF & point )
{
    if (boundingRect().contains(point.toPoint())) {
        return true;
    }
    return false;

}

void VideoFrame::releaseEvent ( QGraphicsSceneMouseEvent * event)
{

    if (event->button() == Qt::LeftButton) {
        rectangle->setVisible(false);
        QRect rect = QRect(topLeft, bottomRight).normalized();

        if (rect.width() > 10 && rect.height() > 10) {
            emit rectangleSelected(rect);
        }
    }
}

QSize VideoFrame::getVideoSize()
{
    return videoSize;
}

void VideoFrame::setTargetRect(QRect rect)
{
    //targetRect->setRect(rect);
    //targetRect->setVisible(true);
    qDebug() << "something something called setTargetRect";
}

QRectF VideoFrame::boundingRect() const
{
    return videoItem->boundingRect();
}

void VideoFrame::setCrosshairPositions(QVector <QPointF> positions)
{
    int i;
    for(i = 0; i < positions.size(); i++) {
        if (i >= crosshairs.size()) {
            return;
        }


        if (!boundingRect().contains(positions.at(i))) {
            continue;
        }

        crosshairs.at(i)->setRect(QRect(positions.at(i).toPoint(), positions.at(i).toPoint() + QPoint(5, 5)));
        crosshairs.at(i)->setVisible(true);
    }
    for(; i < crosshairs.size(); i++) {
        crosshairs.at(i)->setVisible(false);
    }
}

void VideoFrame::disableCrosshair()
{

}

QRect VideoFrame::getRectangle()
{
    return rectangle->rect().toRect();
}





