#ifndef VIDEOFRAME_H
#define VIDEOFRAME_H

#include <QGraphicsItemGroup>
#include <QGraphicsPolygonItem>
#include <QImage>
#include <QPoint>
#include <QGraphicsProxyWidget>

#include <cv.h>

class VideoGraphicsItem;

class VideoFrame : public QObject, public QGraphicsItemGroup
{

    Q_OBJECT

public:
    VideoFrame();
    void pressEvent (QGraphicsSceneMouseEvent *);
    void releaseEvent ( QGraphicsSceneMouseEvent *);
    void moveEvent ( QGraphicsSceneMouseEvent * );
    void setFrameCV(IplImage *);
    QRect getRectangle();
    void setTargetRect(QRect rect);
    void setCrosshairPositions(QVector <QPointF>);
    void disableCrosshair();
    QSize getVideoSize();
    void setGLValue(float);
    VideoGraphicsItem * videoItem;
    bool contains ( const QPointF & point );
    void setPolygon(QPolygon poly);
    void setLine(QLine line);
    QRectF boundingRect() const;
    void setOSD(QString text);

private:
    QGraphicsRectItem frameDecoration;
    QGraphicsRectItem * rectangle;
    QGraphicsRectItem * targetRect;
    QGraphicsLineItem * crosshairH;
    QGraphicsLineItem * crosshairV;
    QGraphicsTextItem osdText;
    QGraphicsTextItem osdTextShadow;
    QGraphicsPolygonItem * polygon;
    QPoint topLeft;
    QPoint bottomRight;
    QVector <QGraphicsRectItem *> crosshairs;
    uchar * data;
    QSize videoSize;
    IplImage * frame;

public slots:
    void setCameraMatrix(CvMat *);

signals:
    void rectangleSelected(QRect);
    void pointSelected(QPoint);
    void mouseMoved(QPoint);



};

#endif // VIDEOFRAME_H
