#ifndef VIDEOGRAPHICSITEM_H
#define VIDEOGRAPHICSITEM_H


#include <QGraphicsItem>
#include <QPainter>
#include <QVector3D>
#include <QGraphicsSceneMouseEvent>
#include <QImage>

#include <cv.h>

/* #include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#define GL_GLEXT_PROTOTYPES
#include <glext.h> */

#include <GL/glew.h>
#ifdef Q_WS_WIN
#include <GL/wglew.h>
#endif

class VideoGraphicsItem : public QGraphicsItem
{
public:
    VideoGraphicsItem(QGraphicsItem *parent);
    QRectF boundingRect() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
    void setFrame(IplImage *);
    void setCameraMatrix(CvMat *);
    void printMatrix(CvMat * L);
    void setDeck(QVector3D pos, QVector3D ang);
    void setPolygon2D(QPolygon polygon);
    void setDeckVisible(bool);
    void setPolygons(QVector <QPolygonF>);
    void setSinglePolygon(QPolygon polygon);

    void setPositions(QVector <QVector3D>);
    void setNormalVector(QVector3D);
    void drawPoint(QVector3D point);
    void setDeckSize(QSize squares, float squareSize);
    void setHighlightedSquare(QPoint square, bool visible);
    void setHighlightedSquareColor(QColor color);
    void setGizmoVisible(bool visible);
    void setLine(QLine);


private:
    GLenum pixelFormat;
    char * imageData;
    GLuint texture;
    GLuint buffer;
    bool glInited;
    bool newImageSize;
    QSize videoSize;
    IplImage * frame;
    int depth;
    QVector3D deckCenter;
    QVector3D deckAngles;
    QVector <QVector3D> positions;
    QVector3D normalVector;
    bool deckVisible;
    QSize squares;
    float squareSize;
    QColor highlightedColor;
    bool frameSet;

    double cparam[3][4];
    GLfloat projMat[16];
    GLfloat modelviewMat[16];

    QVector <QPolygonF> polygons;
    QPolygon polygon2D;
    QPoint highlightedSquare;
    bool highlightSquareVisible;
    bool gizmoVisible;
    float polygonAlpha;
    float polygon2DAlpha;
    float lineAlpha;
    QLine line;
    QLine aftLine;
    float aftLineAlpha;

    QImage qImage;

    void drawGizmo();
    void drawVideo();
    void drawPolygons();
    void drawPolygon2D();
    void drawLine();
    void drawNormalVector();
    void drawDeck(int squaresX, int squaresY, float squareSize);
    int arParamDecompMat( double source[3][4], double cpara[3][4], double trans[3][4] );
    void argConvGLcpara2( double cparam[3][4], int width, int height, double gnear, double gfar, double m[16] );
    void computeProjMat();
    double norm( double a, double b, double c );
    double dot( double a1, double a2, double a3,  double b1, double b2, double b3 );

    // function pointers for PBO Extension
    // Windows needs to get function pointers from ICD OpenGL drivers,
    // because opengl32.dll does not support extensions higher than v1.1.
    /*#ifdef Q_WS_WIN
        PFNGLGENBUFFERSARBPROC pglGenBuffersARB;                     // VBO Name Generation Procedure
        PFNGLBINDBUFFERARBPROC pglBindBufferARB;                     // VBO Bind Procedure
        PFNGLBUFFERDATAARBPROC pglBufferDataARB;                     // VBO Data Loading Procedure
        PFNGLBUFFERSUBDATAARBPROC pglBufferSubDataARB;               // VBO Sub Data Loading Procedure
        PFNGLDELETEBUFFERSARBPROC pglDeleteBuffersARB;               // VBO Deletion Procedure
        PFNGLGETBUFFERPARAMETERIVARBPROC pglGetBufferParameterivARB; // return various parameters of VBO
        PFNGLMAPBUFFERARBPROC pglMapBufferARB;                       // map VBO procedure
        PFNGLUNMAPBUFFERARBPROC pglUnmapBufferARB;                   // unmap VBO procedure
        #define glGenBuffersARB           pglGenBuffersARB
        #define glBindBufferARB           pglBindBufferARB
        #define glBufferDataARB           pglBufferDataARB
        #define glBufferSubDataARB        pglBufferSubDataARB
        #define glDeleteBuffersARB        pglDeleteBuffersARB
        #define glGetBufferParameterivARB pglGetBufferParameterivARB
        #define glMapBufferARB            pglMapBufferARB
        #define glUnmapBufferARB          pglUnmapBufferARB
    #endif */

};

#endif // VIDEOGRAPHICSITEM_H
