#include <QDebug>

#include "videographicsitem.h"
// QGLWidget must be included after videographicsitem because glew.h must be included first.
#include "glwidget.h"
#include "videoscene.h"
#include "videoframe.h"
#include "videodisplay.h"
#include "camera.h"

VideoScene::VideoScene(QWidget * parent) : QGraphicsView(parent)
{
    setBackgroundBrush(QBrush(QColor("black"), Qt::SolidPattern));
    setViewport(new GLWidget(this));
    graphicsScene = new QGraphicsScene();
    setScene(graphicsScene);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    nextLeft = 0;
}


void VideoScene::updateFrames()
{
    int left = 0;
    for(int i = 0; i < videoDisplays.size(); i++) {
        left += videoDisplays.at(i)->updateFrames(left);
        left += 100;
    }
}


void VideoScene::setPolygons(QVector <QPolygonF> polygons)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setPolygons(polygons);
    }
}

void VideoScene::setPositions(QVector <QVector3D> positions)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setPositions(positions);
    }
}

void VideoScene::setDeckPosition(QVector3D pos, QVector3D ang)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setDeck(pos, ang);
    }
}

void VideoScene::setDeckVisible(bool visible)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setDeckVisible(visible);
    }
}

void VideoScene::setNormalVector(QVector3D vec)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setNormalVector(vec);
    }
}

void VideoScene::resizeEvent(QResizeEvent * event)
{
    int maxHeight = 100;
    int width = 0;
    for(int i = 0; i < videoDisplays.size(); i++) {

        QRectF rect = videoDisplays.at(i)->mapToParent(videoDisplays.at(i)->boundingRect()).boundingRect();
        width += rect.width();
        int height = rect.height();

        if (height > maxHeight) {
            maxHeight = height;
        }
    }

    this->resetTransform();

    float scaleX = event->size().width() / (float) width;
    float scaleY = event->size().height() / (float) maxHeight;

    if (scaleX < scaleY) {
        scale(scaleX, scaleX);
    } else {
        scale(scaleY, scaleY);
    }

}

void VideoScene::wheelEvent ( QWheelEvent * event )
{
    if (event->delta() < 0) {
        this->scale(0.8, 0.8);
    } else {
        this->scale(1.2, 1.2);
    }
}

void VideoScene::addCamera(Camera * camera)
{
    VideoDisplay * videoDisplay = new VideoDisplay(camera);
    videoDisplay->getVideoFrame()->setCameraMatrix(camera->getCameraMatrix());
    videoDisplays.append(videoDisplay);
    graphicsScene->addItem(videoDisplay);
    videoDisplay->setPos(nextLeft, 0);
    nextLeft += videoDisplay->boundingRect().width() + 20;
}

void VideoScene::removeCamera(Camera * camera)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        if (videoDisplays.at(i)->getCamera() == camera) {
            graphicsScene->removeItem(videoDisplays.at(i));
            delete videoDisplays.at(i);
            videoDisplays.remove(i);
        }
    }

}

/**
  * Sets number of squares and squaresize in all video frames
  */
void VideoScene::setDeckSize(QSize squares, float squareSize)
{
    for(int i = 0; i < videoDisplays.size(); i++) {
        videoDisplays.at(i)->getVideoFrame()->videoItem->setDeckSize(squares, squareSize);
    }
}



