#ifndef WAVEESTIMATOR_H
#define WAVEESTIMATOR_H

#include <QVector3D>
#include <QSize>
#include <cv.h>

class WaveEstimator
{
public:
    WaveEstimator();
    ~WaveEstimator();
    void setPosition(QVector3D pos,  CvMat *);
    float * getZ();
    float * getAmplitude();
    void setSize(QSize size);
    QSize getSize();
    void allocate();
    float getSquareSize();
    void clear();

private:
    float squareSize;
    QSize size;
    float * zPositions;
    float * zPositionsMin;
    float * zPositionsMax;
    float * zAmplitude;
    float timeConstantSet;
    float timeConstantReset;

    QVector3D position;
    QVector3D angles;
    CvMat * rotMatrix;
    QPointF getPlanePos(QPoint blockPos);
    QVector3D rotateVector(QVector3D qVec, CvMat * rot);
    float computeZ(QPointF planePos);
    void lowpass(float input, float &output, float timeconstant);
    void printMatrix(CvMat * L);

};

#endif // WAVEESTIMATOR_H
